// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	CpBivariate2DInterpolantScattered.h --- Implementation of Interpolant2D class
//
//	DESCRIPTION:
//		Implementation of Interpolant2D class with bivariate interpolant
//		using rectangular grid points - based on routine TOMS 760
//
//	CHANGE HISTORY:
//		UL, 1999-09-08: 1.0.0: - first version
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/slatec/cxx/CpBivariate2DInterpolantRectangular.h,v 1.1 2009/10/16 10:43:14 rsaxton Exp $
//

#ifndef CPBIVARIATE_TWOD_INTERPOLANT_RECTANGULAR_H
#define CPBIVARIATE_TWOD_INTERPOLANT_RECTANGULAR_H

#include "Interpolant2D.h"

namespace Slatec {

template<class NumericType>
class CpBivariate2DInterpolantRectangular :
							public virtual Interpolant2D<NumericType > {
public:
	CpBivariate2DInterpolantRectangular(const std::vector<NumericType>& x,
									  const std::vector<NumericType>& y,
									  const std::vector<NumericType>& z);
		//
		//	PURPOSE:
		//		Instantiate a CpBivariate2DInterpolantRectangular object
		//
		//	ARGUMENTS:
		//		x/y	: vectors of independent variable values
		//		z	: vector of dependent variable values corresponding to
		//			  <x/y>
		//

	NumericType operator()(NumericType x, NumericType y) const;
		//
		//	PURPOSE:
		//		Evaluate interpolation function at a point <x,y>
		//


	~CpBivariate2DInterpolantRectangular();
		//
		//	PURPOSE:
		//		Destroy object instantiated with above constructor(s).
		//

//private:
	NumericType	*_xd, *_yd,		//	grid point values ...
			    *_zd,			//	and function values at those points
				*_wk;			//	work array
	int			_nxd, _nyd;		//	size of _xd/_yd arrays; _zd has to be
								//	of size _nxd*_nyd
};

}

#endif
