// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Curve.cc --- implementation of Curve methods
//
//	DESCRIPTION:
//		This implements all non-inline methods of the Curve class
//
//	CHANGE HISTORY:
//		UL, 2000-03-27: 1.0.0: - first version
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/slatec/cxx/Curve.cc,v 1.1 2000/04/06 14:08:51 gvacanti Exp $
//

#include "Curve.h"
#include "slatec.h"
#include "errstr.h"

extern "C" {

static Slatec::Curve<double>::Functor _DGAUS8_f;

static double Dgaus8_fun(double x)
{
	return (*_DGAUS8_f)(x);
}

FCALLSCFUN1(DOUBLE, Dgaus8_fun, DGAUS8_FUN, dgaus8_fun, DOUBLE);

}

namespace Slatec {

template<class NumericType>
NumericType Curve<NumericType>::integral(NumericType x1, NumericType x2) const
{
	PROC

	double ans,
		   err = 1.e-8,
		   a = x1,
		   b = x2;
	int	   ierr = 0;

	_DGAUS8_f = _f;
	SLATEC_DGAUS8(C_FUNCTION(DGAUS8_FUN, dgaus8_fun), a, b, err, ans, ierr);

	if (ierr < -1 || ierr >= 2)
		errstr<<"Abnormal exit "<<ierr<<" from slatec integration routine"
			  <<Sas::error("NumericalError");

	RETURN ans;
} // Curve<NumericType>::integral()

//
//	explicit template instantiation
//
//template class Curve<float>;
template class Curve<double>;

}
