// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	DataVector --- Collection of utility classes related to data vectors
//
//	DESCRIPTION:
//		A small collection of utility classes around data vectors.
//
//	CHANGE HISTORY:
//		UL, 2001-11-05: 1.0.0: - first version
//	   JBa, 2002-07-17: - made standard compliant
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/slatec/cxx/DataVector.h,v 1.3 2002/07/17 12:09:49 jbakker Exp $
//
#ifndef DATA_VECTORS_H
#define DATA_VECTORS_H

#include <algorithm>
#include <functional>
#include <vector>

namespace DataVector {

template<class T>
bool sorted(const std::vector<T> &v) {
    return !v.empty() ? (std::find_if(v.begin(), v.end(),
                        std::bind2nd(std::less<T>(), v[0])) == v.end()) : true;
}

}

template<class T>
class CorrelatedVectors {
public:
	typedef typename std::vector<T> BaseType;

	CorrelatedVectors(BaseType &free, BaseType &dependent)
		: _free(&free), _dependent(&dependent) {}

	BaseType &free() { return *_free; }
	BaseType &dependent() { return *_dependent; }

	void supplement(const BaseType &extFree, const BaseType &extDep) {
		typename BaseType::const_iterator extFreeBeg = extFree.begin(),
								 extFreeEnd = extFree.end(),
								 extDepBeg = extDep.begin();
		for (typename BaseType::const_iterator i = extFreeBeg; i != extFreeEnd; ++i) {
			unsigned ipos = i-extFreeBeg;
			typename BaseType::iterator freeBeg = _free->begin(),
							   freeEnd = _free->end(),
							   p = lower_bound(freeBeg, freeEnd, *i);
			unsigned pos = p-freeBeg;
			if (p == freeEnd || *p != *i) {
				_free->insert(p, *i);
				_dependent->insert(_dependent->begin()+pos,
								   *(extDepBeg + ipos));
			} else
				*(_dependent->begin()+pos) = *(extDepBeg + ipos);
		}
	}

private:
	BaseType *_free,
			 *_dependent;
};

#endif
