// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	Interpolant2D.h --- A class representing an 2-D interpolation function
//
//	DESCRIPTION:
//		Instances of class Interpolant2D represent 2D-functions which are
//		defined by a discrete set of X/Y/Z data points. The function
//		is evaluated at intermediate data points by interpolation.
//
//	CHANGE HISTORY:
//		UL, 1999-09-08: 1.0.0: - first version
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/slatec/cxx/Interpolant2D.h,v 1.1 2000/01/13 10:19:16 ulammers Exp $
//

#ifndef INTERPOLANT_TWOD_H
#define INTERPOLANT_TWOD_H

#include <vector>

namespace Slatec {

template<class NumericType>
class Interpolant2D {
public:
	Interpolant2D() {}
		//
		//	PURPOSE:
		//		Instantiate an Interpolant2D object
		//

	virtual NumericType operator()(NumericType x, NumericType) const = 0;
		//
		//	PURPOSE:
		//		Evaluate interpolation function at a point <X,Y>
		//

	virtual ~Interpolant2D() {}
		//
		//	PURPOSE:
		//		Destroy object instantiated with above constructor(s).
		//
};

}

#endif
