// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997 - 2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	slatec++.h --- Numerical routines from the SLATEC library in C++
//
//	DESCRIPTION:
//		A namespace Slatec is declared with a set of classes which
//		provide numeric functionality available from the SLATEC library.
//
//	CHANGE HISTORY:
//		GV, 1999-09-08: 1.0.0: - first version; error function added
//		UL, 1999-09-10: 1.1.0: - added CubicSpline/HermiteInterpolant
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/slatec/cxx/slatec++.h,v 1.13 2009/10/16 10:42:54 rsaxton Exp $
//

#ifndef SLATEC_CPP_H
#define SLATEC_CPP_H

#include <Tracer.h>
#include <vector>

#include "Curve.h"
#include "CubicSplineInterpolant.h"
#include "CubicHermiteInterpolant.h"
#include "LinearInterpolant.h"
#include "Bivariate2DInterpolantScattered.h"
#include "Bivariate2DInterpolantRectangular.h"
#include "CpBivariate2DInterpolantRectangular.h"
#include "LinBivariate2DInterpolantRectangular.h"

namespace Slatec {


  // Error function and complementary error function.
  double erf(const double x);
  double erfc(const double x);
  
  // FFT
  std::vector<double> forwardFft(const std::vector<double> & x);
  std::vector<double> backwardFft(const std::vector<double> & x);

  // Convolution
  std::vector<double> convolve(const std::vector<double> & signal,
			  const std::vector<double> & response);


/*    // FFT -- these are provided for completeness. Use the higher level */
/*    // interfaces above if possible. */
/*    void rfftf1(int n, float * c, float * ch, float * wa, int * ifac); */
/*    void rfftb1(int n, float * c, float * ch, float * wa, int * ifac); */
/*    void rffti1(int n, float * wa, int * ifac); */

};  
#endif
