// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// XMM Science Analysis System
//
// Giuseppe Vacanti (European Space Agency) 
// January 24, 2000
//
// Quick and dirty check of the copnvolution.
//
#include "slatec++.h"
#include <cmath>
#include <cassert>
#include <vector>
#include <algorithm>

int main() {

  const double eps = 1e-6;

  const unsigned n = 1024;
  const unsigned m = 32;

  const double s = 1.;
  const double r = 0.5;

  std::vector<double> signal(n);
  std::vector<double> response(n);
  std::vector<double> a(n);

  for(unsigned i = 0; i < n; i++){
    i < 1 ? signal[i] = s : signal[i] = 0;
    i < m ? response[i] = r : response[i] = 0;
  }
  
  a = Slatec::convolve(signal, response); // unnormalized
  for(unsigned i = 0; i < n; i++){
    i < m ? assert(fabs(a[i] - r) < eps) : assert(fabs(a[i]) < eps);
    std::cerr << i << " " << a[i] << std::endl;
  }    
}
