// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Giuseppe Vacanti, XMM Science Operations Team -- September 1999
//

//
// FFT a signal back and forth, to see that the process is reversible.
//

#include "slatec++.h"
#include <cmath>
#include <cassert>
#include <vector>
#include <algorithm>

int main() {

  const float eps = 1e-6;
  const unsigned n = 1024;
  const float p = n;

  std::vector<double> signal(n);
  std::vector<double> a(n);
  std::vector<double> b(n);

  double twopi = acos(-1.) * 2;

  // set up signal array
  for(unsigned i = 0; i < n; i++){
    signal[i] = sin(twopi * i / p);
  }

  a = Slatec::forwardFft(signal); // unnormalized
  b = Slatec::backwardFft(a);
  for(unsigned i = 0; i < n; i++){
    assert(fabs(b[i] / n - signal[i]) < eps);
  }    
}
