// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <iostream>
#include <slatec.h>
#include <cmath>
#include <slatec++.h>
#include <cassert>

int main()
{
  // this could be smaller, but I only have
  // tabulated values with 5 decimal places
  double eps = 1e-5; 
  // x[] is sigma os a normal distribution
  double x[] = {0, 0.5, 1, 1.5, 2, 3, 5};
  // r[] corresponfing integral (-x,x) G(x,0,1): Bevington and Robinson
  double r[] = {0, 0.38292, 0.68269, 0.86638, 0.95449, 0.9973002, 0.99999942657};
  // erf is G(x*sqrt(2),0,1)
  for(unsigned i=0; i<sizeof(x)/sizeof(double); i++){
    double ef = Slatec::erf(x[i]/sqrt(2.));
    double cef= Slatec::erfc(x[i]/sqrt(2.));
    std::cout << ef << " " << cef << "" << r[i] << std::endl;
    assert(fabs(ef - r[i]) <= eps);
    assert(fabs(cef - (1 - r[i])) <= eps);
  }
}
  
