// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// XMM Science Analysis System
//
// Giuseppe Vacanti (European Space Agency) 
// January 20, 2000
//
#include <iostream>
#include <slatec.h>
#include <cmath>
#include <cstdlib>
#include <slatec++.h>
#include <cassert>

int main()
{
  const unsigned N = 100;
  const unsigned NQ = 13;
  const unsigned NW = 19;
  const unsigned NR = static_cast<unsigned>(sqrt(float(N/3)));
  float x[N], y[N], f[N];
//  float rsq[N], a[N];
  int lcell[NR*NR], lnext[N];
  float rsq[N], a[5*N];
  int ier;
  float xmin, ymin, dx, dy, rmax;

  // input grid
  for(unsigned j = 0; j < N; j++){
    x[j] = 0.3*j+static_cast<float>(rand())/RAND_MAX;
    y[j] = 0.6*j;
    f[j] = x[j]+3*y[j];
  }

  TOMS_QSHEP2(N, x, y, f, NQ, NW, NR, lcell, lnext, xmin, ymin, dx, dy, rmax, rsq, a, ier);
  if(ier != 0){
    std::cerr << "ier = " << ier << std::endl;
    exit(1);
  }
  
  float px,py,pz;

  px = py = 1.0;

  TOMS_WRAP_QS2VAL(px, py, N, x, y, f, NR, lcell, lnext, xmin, ymin, dx, dy, rmax, rsq, a, pz);

  std::cout << pz << std::endl;
  px = py = 2.0;

  TOMS_WRAP_QS2VAL(px, py, N, x, y, f, NR, lcell, lnext, xmin, ymin, dx, dy, rmax, rsq, a, pz);

  std::cout << pz << std::endl;
  px = py = 2.1;

  TOMS_WRAP_QS2VAL(px, py, N, x, y, f, NR, lcell, lnext, xmin, ymin, dx, dy, rmax, rsq, a, pz);

  std::cout << pz << std::endl;
}
